#include <cstdlib>
#include <GL/glut.h>

#include "MeshLib/Mesh.h"
#include "MeshLib/Iterators.h"

#include "Geometry/Geometry.h"
#include "Geometry/Quat.h"

void renderScene(void);
void reshapeScene(GLsizei width, GLsizei height);
void setupGLstate();
void mouseClick(int button , int state, int x, int y);
void mouseMove(int x, int y);
void keyBoard(unsigned char key, int x, int y); 

/* rotation quaternion and translation vector for the object */
CQrot       ObjRot(1,0,0,0);
Vector3D    ObjTrans(0,0,0);

/* rotation quaternion and translation vector for the object */
CQrot       EyeRot(1,0,0,0);
Vector3D    EyeTrans(0,0,5);

/* rotation quaternion for the light */
CQrot       LightRot(1,0,0,0);

int win_width, win_height;
int gButton;
float whratio;
int mouseX, mouseY;

Mesh cmesh;

void usage(void)
{
    printf("This program is the first homework from ???\r\n");
    printf("The input is a .m triangle mesh. \r\n");
    printf(" f - flat shading\r\n");
    printf(" s - smooth shading\r\n");
    printf(" w - wireframe\r\n");
    printf(" key ESC - exit\r\n");
}

int main(int argc, char **argv) {
	//1. The first thing you must do is call the function glutInit. It initializes GLUT library.
	glutInit(&argc, argv);

	//2. Define the display mode using the function glutInitDisplayMode(unsigned int mode)
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );


	//3. Define our window
	//3.1. Establish the window's position, i.e. its top left corner. 
	glutInitWindowPosition(100,100);


	//3.2. Choose the window size, using the function glutInitWindowSize(int width, int height).
	int width = 640;
	int height = 480;
	whratio = (float)width /(float)height;
	glutInitWindowSize(width, height);

		
	//3.3 After these settings, we can create the window with a title
	glutCreateWindow("???'s Homework 1");

	//4. Assign a function responsible for the rendering (register a callback)
	glutDisplayFunc(renderScene);

	//5. Assign a reshape function for the "resize" event received by the window.
	glutReshapeFunc(reshapeScene);

	//6. Assign functions for mouse and keyboard events respectively
	glutMouseFunc(mouseClick);
	glutMotionFunc(mouseMove);
	glutKeyboardFunc(keyBoard);

	//7. Some GL initialization
	setupGLstate();

	//8. Load the Mesh
	if (argc!=2)
	{
		printf("need a parameter for the input mesh!");
		exit(0);
	}
	bool flag = cmesh.read(argv[1]);
	if (!flag)
	{
		printf("Fail to read in the mesh %s!", argv[1]); 
		exit(0);
	}
	// may need a object normalization here
	
	//9. Finally, get in the application event processing loop
	glutMainLoop();
	return 1;
}


/* Called when a "resize" event is received by the window. */
void reshapeScene(int width, int height)
{
	win_width=width;
	win_height=height;

	GLsizei mwidth = (width<height*whratio)?width:(height*whratio);

	glViewport(0,0,mwidth, (float)mwidth/whratio);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity(); 

	gluPerspective( 40.0, /* field of view in degrees */
		whratio, /* aspect ratio */
		1.0, /* Z near */ 
		100.0 /* Z far */);

	glMatrixMode(GL_MODELVIEW);

	glutPostRedisplay();
}

void setupGLstate(){

  GLfloat lightOneColor[] = {1, 1, 1, 1};
  GLfloat globalAmb[] = {.1, .1, .1, 1};
  GLfloat lightOnePosition[] = {.0, .2, 1, 0.0};

  glEnable(GL_CULL_FACE);
  glFrontFace(GL_CCW);      
  glEnable(GL_DEPTH_TEST);
  glClearColor(0,0,0,0);
  glShadeModel(GL_SMOOTH);

  glEnable(GL_LIGHT1);
  glEnable(GL_LIGHTING);
  glEnable(GL_NORMALIZE);
  glEnable(GL_COLOR_MATERIAL);

  glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOneColor);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, globalAmb);
  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

  glLightfv(GL_LIGHT1, GL_POSITION, lightOnePosition);
}

void  mouseClick(int button , int state, int x, int y)
{  
  if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) 
  { 
	  gButton = GLUT_LEFT_BUTTON;  
	  //blahblah
  }
  
  if (button == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN) {
      gButton = GLUT_MIDDLE_BUTTON;
	  mouseX = x;
      mouseY = y;      
   }
  
  if (button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN) {
      gButton = GLUT_RIGHT_BUTTON;
	  mouseX = x;
      mouseY = y;
   }
  return ;
}

void mouseMove(int x, int y)
{
  Vector3D trans;
  
  /* rotation*/
  if (gButton == GLUT_LEFT_BUTTON ) 
  {
	//blahblah
	glutPostRedisplay();
  }
  
  /*xy translation */
  if (gButton == GLUT_MIDDLE_BUTTON) 
  {
	  //blahblah
      glutPostRedisplay();
  }
  
  /* zoom in and out */
  if (gButton == GLUT_RIGHT_BUTTON ) 
  {
      double scale = 10./win_height;
      trans =  Vector3D(0,0, scale*(mouseY-y)   );
	  mouseX = x;
	  mouseY = y;
	  ObjTrans = ObjTrans-trans;
	  //or you can do : EyeTrans = EyeTrans+trans;
      glutPostRedisplay();
  }
}


void keyBoard(unsigned char key, int x, int y) 
{	
	switch( key )
	{
	case 'f':
		//Flat Shading
		glPolygonMode(GL_FRONT, GL_FILL);
		break;
	case 's':
    //Smooth Shading
		glPolygonMode(GL_FRONT, GL_FILL);
		break;
	case 'w':
	  //Wireframe mode
		glPolygonMode(GL_FRONT, GL_LINE);
		break;
	case '?':
		//usage
		usage();
		break;  
	case 27:
		exit(0);
		break;
	}
  glutPostRedisplay();  
}




/* setup the object, transform from the world to the object coordinate system */
void setupObject(void)
{
	double rot[16];
	glTranslated( ObjTrans.x, ObjTrans.y, ObjTrans.z);
	ObjRot.convert( rot );
	glMultMatrixd(( GLdouble *)  rot );	
}

/* the eye is always fixed at world z = +5 */
void setupEye(void)
{
  glLoadIdentity();

  double rot[16];
  CQrot eye = EyeRot;
  eye^(-1);
  eye.convert( rot );
   
  glTranslated( -EyeTrans.x, -EyeTrans.y, -EyeTrans.z); 
  glMultMatrixd(( GLdouble *)  rot );
}

/* setup light */
void setupLight()
{
  Vector3D position(0,0,1);
  position = LightRot* position;
  GLfloat lightOnePosition[4]={position.x, position.y, position.z, 0};
  glLightfv(GL_LIGHT1, GL_POSITION, lightOnePosition);
}

void renderScene(void) 
{
	/* clear frame buffer */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	  
    setupLight();
	/* transform from the eye coordinate system to the world system */
	setupEye();
	glPushMatrix();
	/* transform from the world to the ojbect coordinate system */
	setupObject();
  
	//Draw the mesh;
	glBegin(GL_TRIANGLES);
	for (MeshFaceIterator fit(&cmesh); !fit.end(); ++fit)
	{
		Face * f = *fit;
		HalfEdge * he = f->halfedge();
		Point * pt[3];
		pt[0] = &he->source()->point();
		pt[1] = &he->target()->point();
		pt[2] = &he->he_next()->target()->point();
		glVertex3f((*pt[0])[0], (*pt[0])[1],(*pt[0])[2]);
		glVertex3f((*pt[1])[0], (*pt[1])[1],(*pt[1])[2]);
		glVertex3f((*pt[2])[0], (*pt[2])[1],(*pt[2])[2]);
	}
	glEnd();

	glPopMatrix();
	glutSwapBuffers();
}
